
public class BSTUtils<E> {

	public static <E> void inorder(BTNode<E> root, BTNode.Visitor<E> visitor) {
		visitor.beforeVisit();
		if (root != null) {
			inorder(root.left, visitor);
		}
		visitor.visit(root);
		if (root != null) {
			inorder(root.right, visitor);
		}
		visitor.afterVisit();
	}

	public static <E> void reverseInorder(BTNode<E> root, BTNode.Visitor<E> visitor) {
		visitor.beforeVisit();
		if (root != null) {
			reverseInorder(root.right, visitor);
		}
		visitor.visit(root);
		if (root != null) {
			reverseInorder(root.left, visitor);
		}
		visitor.afterVisit();
	}

	public static <E> void preorder(BTNode<E> root, BTNode.Visitor<E> visitor) {
		visitor.visit(root);
		if (root != null) {
			preorder(root.left, visitor);
			preorder(root.right, visitor);
		}
		visitor.afterVisit();
	}

	public static <E> void postorder(BTNode<E> root, BTNode.Visitor<E> visitor) {
		visitor.beforeVisit();
		if (root != null) {
			postorder(root.left, visitor);
			postorder(root.right, visitor);
		}
		visitor.visit(root);
		visitor.afterVisit();
	}

	public static <E> BTNode<E> copy(BTNode<E> root) {
		if (root == null) {
			return null;
		}
		return new BTNode<E>(root.data, copy(root.left), copy(root.right));
	}

	public static <E> long size(BTNode<E> root) {
		if (root == null) {
			return 0;
		}
		return 1 + size(root.left) + size(root.right);
	}

	public static <E> boolean isLeaf(BTNode<E> node) {
		return node.left == null && node.right == null;
	}

	public static <E> int children(BTNode<E> node) {
		return (node.left != null ? 1 : 0) + (node.right != null ? 1 : 0);
	}

	public static class SimplePrinter<E> extends BTNode.Visitor<E> {
		@Override
		public void visit(BTNode<E> node) {
			if (node != null) {
				System.out.print(node.data);
			}
		}
	}

	public static class StringPrinter<E> extends BTNode.Visitor<E> {

		private StringBuilder buffer;

		public StringPrinter() {
			buffer = new StringBuilder();
		}

		@Override
		public void visit(BTNode<E> node) {
			if (node != null) {
				buffer.append(node.data.toString());
			}
		}

		public String toString() {
			return buffer.toString();
		}
	}

	public static class TreePrinter<E> extends BTNode.Visitor<E> {

		private int depth = 0;
		private static final int DIFF = 4;

		private void printMany(char c) {
			for (int i = 0; i < depth; ++i) {
				System.out.print(c);
			}
		}

		@Override
		public void visit(BTNode<E> node) {
			printMany(' ');
			if (node == null) {
				System.out.println('-');
			} else {
				System.out.println(node.data);
			}
		}

		@Override
		public void beforeVisit() {
			depth += DIFF;
		}

		@Override
		public void afterVisit() {
			depth -= DIFF;
		}

	}
}
